#include <fstream>
#include <iostream>
#include <vector>
#include <algorithm>
#include <math.h>
#include "revKSketch.h"
#include "revHash.h"
#include "globals.h"
#include "hash.h"

using namespace std;

typedef struct record_s {
	uint32 src_ip;
	uint32 dst_ip;
	uint32 syn:1;
	uint32 fin:1;
	uint32 ack:1;
	uint32 rst:1;
	uint32 psh:1;
	uint32 urg:1;
	uint32 value:26;
} record_t;

static unsigned long long totalitems;

void printip(unsigned int ip)
{
	printf("%d.%d.%d.%d", (ip >> 24) & 0xff, (ip >> 16) & 0xff, (ip >> 8) & 0xff, ip & 0xff);
}

void Load_flow(char *filename, RevKSketch * sketch)
{
	fstream f;
	record_t rec;

	// open the filename
	f.open(filename, ios::in | ios::binary);
	if (!f) {
		cerr << "Error opening the file " << filename << endl;
	}


	if (ipflag = SRCIP_DETECTION) {
		while (f.read((char *) &rec, sizeof(rec))) {
			if (rec.syn && !rec.ack) {
				sketch->AddValue(rec.src_ip, 1);
			} else if (rec.syn && rec.ack) {
				sketch->AddValue(rec.src_ip, -1);
			}
			totalitems++;
		}
	} else if (ipflag == DSTIP_DETECTION) {
		while (f.read((char *) &rec, sizeof(rec))) {
			if (rec.syn && !rec.ack) {
				sketch->AddValue(rec.dst_ip, 1);
			} else if (rec.syn && rec.ack) {
				sketch->AddValue(rec.dst_ip, -1);
			}
			totalitems++;
		}
		
	}


	f.close();

}


int main(int argc, char **argv)
{
	list < flow_t > flow1, flow2;

	if (argc != 8) {
		cerr <<
		    "Usage: sketch_test filename threshold R <num tables>"
		    << " <index bits> <iteration size> <original num tables> <original idx bits>" <<
		    endl;
		exit(-1);
	}

	int threshold = atoi(argv[2]);
	int r = atoi(argv[3]);
	int numTbls = atoi(argv[4]);
	int inxBits = atoi(argv[5]);
	int iterSize = atoi(argv[6]);
	int OnumTbls = atoi(argv[7]);
	int OinxBits = atoi(argv[8]);

	RevKSketch *Sketch1 = new RevKSketch(numTbls, r, inxBits, iterSize, OnumTbls, OinxBits);
	RevKSketch *Sketch2 = new RevKSketch(numTbls, r, inxBits, iterSize, OnumTbls, OinxBits);

	double startTime, stopTime, insertTime, cdTime;

	startTime = current_time();
	Load_flow(argv[1], Sketch2);
	stopTime = current_time();
	insertTime = stopTime - startTime;

	startTime = current_time();
	list < KeyType > offendingKeys;
	Sketch2->ChangeDetection(offendingKeys, *Sketch1, threshold);
	stopTime = current_time();
	cdTime = stopTime - startTime;

	cerr << "InsertTime: " << insertTime << endl;
	cerr << "DetectionTime: " << cdTime << endl;
	cerr << "TotalItems: " <<(totalitems) << endl;

	for (list < KeyType >::iterator listIt = offendingKeys.begin();
	     listIt != offendingKeys.end(); listIt++) {
		cout << *listIt << endl;
	}

	delete testSketch1;
	delete testSketch2;

	return 0;
}
