#include <fstream>
#include <iostream>
#include <vector>
#include <algorithm>
#include <math.h>
#include "revKSketch.h"
#include "revHash.h"
#include "globals.h"
#include "hash.h"

using namespace std;

typedef struct record_s {
  uint32 src_ip;
  uint32 dst_ip;
  uint32 syn:1;
  uint32 fin:1;
  uint32 ack:1;
  uint32 rst:1;
  uint32 psh:1;
  uint32 urg:1;
  uint32 value:26;
} record_t;

typedef struct flow_r {
	uint32 key;
	uint32 value;
	} flow_t;

void printip(unsigned int ip)
{
	printf("%d.%d.%d.%d",(ip>>24)&0xff,(ip>>16)&0xff,(ip>>8)&0xff,ip&0xff);
}
 
void Load_flow(char * filename, list<flow_t> &flows)
{
    fstream f;

#ifdef BINARY_INPUT // direct input in binary format

#define REC_BUFF_SIZE 256
    
    record_t rec;
    flow_t tflow;
    
    // open the filename
    f.open(filename,ios::in | ios::binary);
    if(!f)
    {
        cerr << "Error opening the file " << filename << endl;
    }

    while(f.read((char*)&rec,sizeof(rec))) {
      tflow.key=rec.src_ip;
      tflow.value=rec.value;
      flows.push_back(tflow);
    }
    
    f.close();
    
#else
    
    char buffer[255];
    char arg1[32],arg2[32],arg3[32],arg4[32],arg5[32],arg6[32];
    char ip1[4],ip2[4],ip3[4],ip4[4];
    unsigned int ip;

    // open the filename
    f.open(filename,ios::in);
    if(!f)
    {
        cerr << "Error opening the file " << filename << endl;
    }

    string sep=" ";
    while(f.getline(buffer,255)) {
      sscanf(buffer,"%s %s %s %s %s %s",arg1,arg2,arg3,arg4,arg5,arg6);
      sscanf(arg1,"%[^.].%[^.].%[^.].%[^.]",ip1,ip2,ip3,ip4);
      ip=(atoi(ip1) << 24) + 
	(atoi(ip2) << 16) + 
	(atoi(ip3) << 8) + 
	(atoi(ip4));

      // arg1 is the source ip address
      // arg4 is the packet size
      tflow.key=ip;
      tflow.value=atoi(arg4);
      flows.push_back(tflow);
    }
    
    f.close();

#endif
}



void parse_flow(list<flow_t> flows, RevKSketch *sketch)
{

	for (list<flow_t>::iterator fit = flows.begin(); fit != flows.end(); fit++) {
		sketch->AddValue(fit->key, fit->value);
     	}
}




/*
    fstream f;



#ifdef BINARY_INPUT // direct input in binary format

#define REC_BUFF_SIZE 256
    
    record_t rec;
    
    // open the filename
    f.open(filename,ios::in | ios::binary);
    if(!f)
    {
        cerr << "Error opening the file " << filename << endl;
    }

    while(f.read((char*)&rec,sizeof(rec)))
      sketch->AddValue(rec.src_ip, rec.value);
    
    f.close();
    
#else
    
    char buffer[255];
    char arg1[32],arg2[32],arg3[32],arg4[32],arg5[32],arg6[32];
    char ip1[4],ip2[4],ip3[4],ip4[4];
    unsigned int ip;

    // open the filename
    f.open(filename,ios::in);
    if(!f)
    {
        cerr << "Error opening the file " << filename << endl;
    }

    string sep=" ";
    while(f.getline(buffer,255)) {
      sscanf(buffer,"%s %s %s %s %s %s",arg1,arg2,arg3,arg4,arg5,arg6);
      sscanf(arg1,"%[^.].%[^.].%[^.].%[^.]",ip1,ip2,ip3,ip4);
      ip=(atoi(ip1) << 24) + 
	(atoi(ip2) << 16) + 
	(atoi(ip3) << 8) + 
	(atoi(ip4));

      // arg1 is the source ip address
      // arg4 is the packet size
      sketch->AddValue(ip,atoi(arg4)); 
    }
    
    f.close();

#endif
} */

int main(int argc, char **argv)
{
  list<flow_t> flow1,flow2;

  if( argc != 8 ) {
    cerr << "Usage: sketch_test flow1 flow2 threshold R <num tables>" 
	 << " <index bits> <iteration size>" << endl;
    exit(-1);
  }

  int threshold = atoi(argv[3]);
  int r = atoi(argv[4]);
  int numTbls = atoi(argv[5]);
  int inxBits = atoi(argv[6]);
  int iterSize = atoi(argv[7]);

  cout << threshold << endl
       << r         << endl
       << numTbls   << endl
       << inxBits   << endl
       << iterSize  << endl;

  RevKSketch *testSketch1 = new RevKSketch(numTbls, r, inxBits, iterSize,numTbls,inxBits);
  RevKSketch *testSketch2 = new RevKSketch(numTbls, r, inxBits, iterSize,numTbls,inxBits);

  double startTime, stopTime, insertTime, cdTime;
 
  Load_flow(argv[1],flow1);
  Load_flow(argv[2],flow2); 

  startTime = current_time();
  parse_flow(flow1, testSketch1);
  parse_flow(flow2, testSketch2);
  stopTime = current_time();
  insertTime = stopTime - startTime;

  startTime = current_time();
  list<KeyType> offendingKeys;
  testSketch2->ChangeDetection(offendingKeys, *testSketch1, threshold);
  stopTime = current_time();
  cdTime = stopTime - startTime;

  cout << insertTime << endl;
  cout << cdTime     << endl;
  cout << (flow1.size()+flow2.size()) << endl;

  for( list<KeyType>::iterator listIt = offendingKeys.begin();                
       listIt != offendingKeys.end();
       listIt++ ) {
    cout << *listIt << endl;
  }

  /*
  // Test basic original sketch functions on a big endian machine
  testSketch1->extraSketch->AddValue(0x12345678U, 17);
  cout << "Expect 17, got " 
       << testSketch1->extraSketch->EstValue(0x12345678U) << endl;
  
  uint32 temp = testSketch1->mangler->MangleCharTable(1109640983U);
  cout << "Mangled value for " << hex << 1109640983U << " = " << hex
       << temp << endl;
  
  cout << "Reversed value = " << hex
       << testSketch1->mangler->ReverseCharTable(temp) << endl;

  for( int i = 0; i < numTbls; i++ ) {
    cout << i << ": index = " 
	 << testSketch1->GetIndex(1109640983, i) << endl;
  }
  */

  delete testSketch1;
  delete testSketch2;

  return 0;
}
