/* -*-  Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
// $Header: /home/zhichun/Research/CVS/RevSketch/util.h,v 1.1.1.1 2005/01/13 17:22:28 zhichun Exp $

#ifndef util_h
#define util_h

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "globals.h"

const double E = 2.71828182845904523536028747;
const double PI = 3.14159265358979323846264338;

typedef unsigned long long  uint64;
typedef unsigned int        uint32;
typedef unsigned short      uint16;
typedef unsigned char       uint8;

const uint64 MAXUINT64 = (uint64) (-1);
const uint32 MAXUINT32 = (uint32) (-1);
const uint16 MAXUINT16 = (uint16) (-1);
const uint8  MAXUINT8  = (uint8)  (-1);

typedef ValueType val_t;

typedef union {
	uint64 as_int64; 
	uint32 as_int32s[2];
	uint16 as_int16s[4];
	uint8  as_int8s[8];
} int64views;

typedef union {
	uint32 as_int32; 
	uint16 as_int16s[2];
	uint8  as_int8s[4];
} int32views;

extern uint32 rand32bit();
extern uint64 rand64bit();

extern void message(const char* msg);
extern void message(const char* fmt, const char* msg);
extern void internal_error(const char* msg);
extern void internal_error(const char* fmt, const char* arg);

inline val_t max(val_t a, val_t b)
	{
	return a > b ? a : b;
	}

inline val_t min(val_t a, val_t b)
	{
	return a < b ? a : b;
	}

inline void swap(val_t &a, val_t &b)
	{
	val_t temp;
	temp = a; a = b; b = temp;
	}

inline uint32 smartdietz(uint32 key, uint32 mask, uint64 a, uint64 b)
	{
	int64views h;
	h.as_int64 = a * key + b;
#ifdef BIGENDIAN      
	return h.as_int32s[0] & mask;
#else
	return h.as_int32s[1] & mask;
#endif
	}

inline uint32 smartdietz(uint64 key, uint32 mask,
			 uint64 a0, uint64 b0,
			 uint64 a1, uint64 b1)
	{
	int64views h;
	h.as_int64 =
		(a0 * (key>>32) + b0) ^ 
		(a1 * (key&0xffffffff) + b1);
#ifdef BIGENDIAN      
	return h.as_int32s[0] & mask;
#else
	return h.as_int32s[1] & mask;
#endif
	}

extern double current_time();
extern val_t quickmedian(val_t *a, int n);

#endif
