#include <fstream>
#include <iostream>
#include <vector>
#include <map>

using namespace std;

typedef struct record_s {
  unsigned int src_ip;
  unsigned int dst_ip;
  unsigned int value;
} record_t;

void printip(unsigned int ip)
{
	printf("%d.%d.%d.%d",(ip>>24)&0xff,(ip>>16)&0xff,(ip>>8)&0xff,ip&0xff);
}
 
void construct_table(char* filename,map<unsigned int,unsigned int>& table,
		     int flag)
{
#ifdef  BINARY_INPUT // direct input in binary format
  ifstream f;
  record_t rec;
  long long numBytes = 0;

  // open the filename
  
  f.open(filename,ios::in | ios::binary);
  if(!f)
    {
      cerr << "Error opening the file " << filename << endl;
    }

  while(f.read((char*)&rec,sizeof(rec))) {
    rec.value*=flag;
    table[rec.src_ip] += rec.value;
    numBytes  += rec.value;

    printf("DEBUG: SRC:");
    printip(rec.src_ip);
    printf(" DEST:");
    printip(rec.dst_ip);
    printf("      Value=%d\n",rec.value);
  }

  f.close();
  
#else
  char buffer[355];
  char arg1[32],arg2[32],arg3[32],arg4[32],arg5[32],arg6[32];
  char ip1[4],ip2[4],ip3[4],ip4[4];
  unsigned int ip;
  ifstream f;
  long long numBytes = 0;

  
  // open the filename
  f.open(filename,ios::in);
  if(!f) {
    cerr << "Error opening the file " << filename << endl;
    return;
  }

  cerr << "file opened succesfully !" << flush << endl;
  string sep=" ";
  while(f.getline(buffer,354)) {
    sscanf(buffer,"%s %s %s %s %s %s",arg1,arg2,arg3,arg4,arg5,arg6);
    sscanf(arg1,"%[^.].%[^.].%[^.].%[^.]",ip1,ip2,ip3,ip4);
    ip = (atoi(ip1) << 24) + 
      (atoi(ip2) << 16) + 
      (atoi(ip3) << 8) + 
      (atoi(ip4));

    // arg1 is the source ip address
    // arg4 is the packet size
    // add the value to the map
    table[ip] += flag*atoi(arg4);
    numBytes  += flag*atoi(arg4);
  }
  
  f.close();
#endif
  
}

void report_differences(map<unsigned int,unsigned int>& table,
		//	map<unsigned int,unsigned int>& table2, 
			int threshold)
{
  // iterate through the second table and find the difference from the second
  int difference;
  int diffcount=0;
  
  for (map<unsigned int,unsigned int>::const_iterator iterator=table.begin();
       iterator!=table.end();
       ++iterator) {
   
    difference=iterator->second;;
#ifdef ABSOLUTE_CHANGE
    if(difference<0) difference=-difference;
#endif
    if(difference>threshold) {
      cout << iterator->first <<  " " << difference << endl;
      diffcount++;
    }
  }
}

int main(int argc,char* argv[])
{
  map<unsigned int, unsigned int> *costTbl1, *costTbl2;

  costTbl1 = new map<unsigned int, unsigned int>;
  costTbl2 = new map<unsigned int, unsigned int>;

  if(argc<4) {
    cerr << "usage: groundt tracefile1 tracefile2 threshold" << endl;
    exit(1);
  } 

  int threshold = atoi(argv[3]);
  construct_table(argv[1],*costTbl1,-1);
  construct_table(argv[2],*costTbl1,+1);

  report_differences(*costTbl1,threshold);

  delete costTbl1;
  delete costTbl2;
  
  return 0;
}
