#include <fstream>
#include <iostream>
#include <vector>
#include <map>

#include "kSketch.h"
#include "hash.h"

using namespace std;

typedef struct record_s {
  unsigned int src_ip;
  unsigned int dst_ip;
  unsigned int value;
} record_t;

	
void construct_table(char* filename,map<unsigned int,unsigned int>& table,
		     KSketch *tsketch)
{
#ifdef  BINARY_INPUT // direct input in binary format
  ifstream f;
  record_t rec;
  long long numBytes = 0;

  // open the filename
  
  f.open(filename,ios::in | ios::binary);
  if(!f)
    {
      cerr << "Error opening the file " << filename << endl;
    }

  while(f.read((char*)&rec,sizeof(rec))) {
    tsketch->AddValue(rec.src_ip,rec.value);	
    table[rec.src_ip] += rec.value;
    numBytes  += rec.value;
  }

  f.close();
  
#else
  char buffer[355];
  char arg1[32],arg2[32],arg3[32],arg4[32],arg5[32],arg6[32];
  char ip1[4],ip2[4],ip3[4],ip4[4];
  unsigned int ip,value;
  ifstream f;
  long long numBytes = 0;

  
  // open the filename
  f.open(filename,ios::in);
  if(!f) {
    cerr << "Error opening the file " << filename << endl;
    return;
  }

  cerr << "file opened succesfully !" << flush << endl;
  string sep=" ";
  while(f.getline(buffer,354)) {
    sscanf(buffer,"%s %s %s %s %s %s",arg1,arg2,arg3,arg4,arg5,arg6);
    sscanf(arg1,"%[^.].%[^.].%[^.].%[^.]",ip1,ip2,ip3,ip4);
    ip = (atoi(ip1) << 24) + 
      (atoi(ip2) << 16) + 
      (atoi(ip3) << 8) + 
      (atoi(ip4));

    // arg1 is the source ip address
    // arg4 is the packet size
    // add the value to the map
    value=atoi(arg4);
    tsketch->AddValue(ip,value);
    table[ip] += atoi(value);
    numBytes  += atoi(value);
  }
  
  f.close();
#endif
  
}

void report_differences(map<unsigned int,unsigned int>& table1,
			map<unsigned int,unsigned int>& table2, 
			int threshold)
{
  // iterate through the second table and find the difference from the second
  int difference;
  int diffcount=0;
  
  for (map<unsigned int,unsigned int>::const_iterator iterator=table2.begin();
       iterator!=table2.end();
       ++iterator) {
   
    difference=iterator->second-table1[iterator->first];
#ifdef ABSOLUTE_CHANGE
    if(difference<0) difference=-difference;
#endif
    if(difference>threshold) {
      cout << iterator->first <<  " " << difference << endl;
      diffcount++;
    }
  }
}

void output_diff(char *fname, map<unsigned int,unsigned int>& table1, 
                 map<unsigned int,unsigned int>& table2,
                 KSketch *tsketch1, KSketch *tsketch2)
{
  ifstream f;
  unsigned int ip;
  int tvalue;
  double value;

  // open the filename
  f.open(fname,ios::in);
  if(!f) {
    cerr << "Error opening the file " << fname << endl;
    return;
  }

  while( f >> ip ) {
	cout << ip << '\t';
	tvalue= (table2[ip] - table1[ip]);
	cout << tvalue << '\t';
	cout << tsketch1->EstValue(ip) <<'\t' << tsketch2->EstValue(ip) <<'\t';
        value=tsketch2->EstValue(ip) - tsketch1->EstValue(ip);
	cout << (value) <<'\t';
        cout << (value/(double)tvalue) << endl;
  }

}
int main(int argc,char* argv[])
{
  map<unsigned int, unsigned int> *costTbl1, *costTbl2;

  costTbl1 = new map<unsigned int, unsigned int>;
  costTbl2 = new map<unsigned int, unsigned int>;

  

  if(argc<6) {
    cerr << "usage: groundt tracefile1 tracefile2 iplistfile numTbls inxbits" << endl;
    exit(1);
  } 
  
  int numTbls=atoi(argv[4]);
  int inxbits=atoi(argv[5]);
  KSketch *sketch1 = new KSketch(numTbls,inxbits);
  KSketch *sketch2 = new KSketch(numTbls,inxbits);

  construct_table(argv[1],*costTbl1,sketch1);
  construct_table(argv[2],*costTbl2,sketch2);

  output_diff(argv[3],*costTbl1,*costTbl2,sketch1,sketch2);

  delete costTbl1;
  delete costTbl2;

  delete sketch1;
  delete sketch2;
  
  return 0;
}
